;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;;                    Recursive version of Pascal's triangle:
;; 
;; Andy Long, spring 2022
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; We need to create the base case -- a list containing a single "1", which
;; constitutes row 0 of Pascal's triangle:
(setq row0 '(1))

;; Okay: now, onto the function which embeds the recursive or inductive step:
(defun pascalRow(n &key verbose)
  "
;; Examples: 
 (pascalRow 13 :verbose t)
 (trace pascalRow)
 (pascalRow 13)
"
  (if (not (and (integerp n) (>= n 0))) (error "Only non-negative integers are allowed"))
  (let ((result
	 (case n
	       ;; this is the base case:
	       (0 row0)
	       ;; and, if we're not in a base case, then we should use recursion. 
	       ;; This means that function pascalRow invokes itself, on the
	       ;; previous integer::
	       (t (let ((previous (pascalRow (- n 1) :verbose verbose)))
		    ;; The next row is the sum of the previous row with itself,
		    ;; but shifted (which I've carried out by the appropriate
		    ;; padding of the previous row):
		    (+ (combine 0 previous) (combine previous 0))
		    )
		  )
	       )
	 )
	)
    (if verbose (print result))
    result
    )
  )
(try-out pascalrow)
